<?php
(defined('_JEXEC') || PHP_SAPI == 'cli') or die('Restricted access');

// Checking PHP Version
if (version_compare(PHP_VERSION, '7.4.0', '<')) {
    die('Payment module "Payment system VTB" requires PHP version 7.4.0 or higher.');
}

require_once __DIR__ . '/vendor/autoload.php';

use Vtbpay\Classes\VtbApi;

class pm_vtbpay extends PaymentRoot
{
    /**
     * The constructor initializes the payment root, language file, and logger.
     */
    public function __construct()
    {
        JSFactory::loadExtLanguageFile('pm_vtbpay');
        JLog::addLogger(
            ['text_file' => 'com_jshopping.pm_vtbpay.php'],
            JLog::ALL,
            ['pm_vtbpay']
        );
    }


    /**
     * Displays the payment form for the VTB Pay payment method.
     *
     * @param array $params The parameters for the payment form.
     * @param array $pmconfigs The payment method configurations.
     */
    public function showPaymentForm($params, $pmconfigs)
    {
        include dirname(__FILE__) . '/paymentform.php';
    }


    /**
     * Displays the admin form parameters for the VTB Pay payment method plugin.
     *
     * @param array $params The parameters for the admin form.
     */
    public function showAdminFormParams($params)
    {
        $path_current_payment = str_replace(JPATH_ROOT, '', __DIR__);
        $document = JFactory::getDocument();
        $document->addScript($path_current_payment . '/assets/js/admin_vtbpay.js');
        
        $module_params = [
            'vtbpay_client_id',
            'vtbpay_client_secret',
            'vtbpay_merchant_authorization',
            'vtbpay_test_mode',
            'transaction_end_status'
        ];
        foreach ($module_params as $module_param) {
            if (!isset($params[$module_param])) $params[$module_param] = '';
        }

        if (empty($params['vtbpay_test_mode'])) {
            $params['vtbpay_test_mode'] = 0;
        }

        $orders = JModelLegacy::getInstance('orders', 'JshoppingModel');
        include dirname(__FILE__) . '/adminparamsform.php';

    }
    
    
    /**
     * Creates a new VtbApi instance with the given payment method configurations.
     *
     * @param array $pm_config The payment method configurations.
     * @return VtbApi The new VtbApi instance.
     */
    protected function getVtbApi($pm_config)
    {
        return new VtbApi(
            $pm_config['vtbpay_client_id'],
            $pm_config['vtbpay_client_secret'],
            $pm_config['vtbpay_test_mode'],
            $pm_config['vtbpay_merchant_authorization']
        );
    }
    
    
    /**
     * Generates the payment URL for the given order using the VTB Pay API.
     *
     * @param array $pm_config The payment method configurations.
     * @param object $order The order object.
     * @return string The payment URL.
     * @throws Exception If there is an error fetching the payment URL.
     */
    public function getOrderLink($pm_config, $order)
    {
        $order_id = $order->order_id;

        $return_url = JURI::root() . 'index.php?option=com_jshopping&controller=checkout&task=step7&js_paymentclass=' . __CLASS__ . '&act=finish';

        $response = $this->getVtbApi($pm_config)->getOrderLink(
            $order_id,
            $order->email,
            time(),
            $order->order_total,
            $return_url
        );

        if($pm_config['vtbpay_logging']) $this->vtbpayLogger(__FUNCTION__ . ' > getOrderLink. ', JLog::DEBUG, [
            'arguments' => [
                'order_id' => $order_id, 
                'email' => $order->email,
                'time' => time(), 
                'order_total' => $order->order_total, 
                'return_url' => $return_url
            ],
            'response' => $response
        ]);

        if (
            isset($response['object']['payUrl']) &&
            !empty($payUrl = $response['object']['payUrl'])
        ) {
            return $payUrl;
        } 
        else {
            throw new \Exception(_JSHOP_CFG_VTBPAY_MSG_URL_FETCH_ERROR . ' Order ID: ' . $order_id);
        }
    }


    /**
     * Redirects the user to the payment URL for the given order.
     *
     * @param array $pm_config The payment method configurations.
     * @param object $order The order object.
     */
    public function showEndForm($pm_config, $order)
    {
        try {
            $paymentUrl = $this->getOrderLink($pm_config, $order);
            header('Location: ' . $paymentUrl);
            exit;
        } catch (\Exception $e) {
            $this->vtbpayLogger(sprintf(__FUNCTION__ . ' > VtbPay Exception: %s', $e->getMessage()), JLog::ERROR);
            echo $e->getMessage();
        }
    }


    /**
     * Checks the transaction status for the given order and response code.
     *
     * @param array $pm_config The payment method configurations.
     * @param object $order The order object.
     * @param string $rescode The response code.
     * @return array An array containing the transaction status and a message.
     */
    public function checkTransaction($pm_config, $order, $rescode)
    {
        $order_id = $order->order_id;

        try {
            if($order->order_status == $pm_config['transaction_end_status']) {
                throw new \Exception(_JSHOP_CFG_VTBPAY_MSG_ORDER_ALREADY_PAID);
            }
          
            $response = $this->getVtbApi($pm_config)->getOrderInfo($order_id);
            
            if($pm_config['vtbpay_logging']) $this->vtbpayLogger(__FUNCTION__ . ' > getOrderInfo. ', JLog::DEBUG, [
                'arguments' => [
                    'order_id' => $order_id
                ],
                'response' => $response
            ]);

            $orderStatus = $response['object']['status']['value'] ?? '';

            if ($orderStatus !== 'PAID') {
                throw new \Exception(_JSHOP_CFG_VTBPAY_MSG_TRANSACTION_REJECTED);
            }
            
            $msg = _JSHOP_CFG_VTBPAY_MSG_SUCCESSFULLY_PAID . ' Order ID: ' . $order_id;
            JFactory::getApplication()->enqueueMessage($msg);
            return [1, $msg];
            
        } catch (\Exception $e) {
            $msg = $e->getMessage() . ' Order ID: ' . $order_id; 
            $this->vtbpayLogger(sprintf(__FUNCTION__ . ' > VtbPay Exception: %s', $msg), JLog::ERROR);
            return [0, $msg];
        }
    }


    /**
     * Retrieves the URL parameters for the payment method.
     *
     * @param array $config The payment method configurations.
     * @return array An array containing the URL parameters.
     */
    public function getUrlParams($config)
    {
        $params = [];
        $params['order_id'] = JFactory::$application->input->getString('orderId', null);
        $params['hash'] = '';
        $params['checkHash'] = 0;
        $params['checkReturnParams'] = 1;
        return $params;
    }
    
    
    /** 
     * Logs a message with optional context. Log type can be either DEBUG or ERROR.
     * 
     * @param string $message The message to log
     * @param string $type The type of log. Can be either DEBUG or ERROR
     * @param array $context An optional array of context data to log with the message
     * @param bool $print_r If set to true, the context array will be printed using print_r instead of json_encode
     * @return void
     */
    private function vtbpayLogger(
        $message, 
        $type = '',
        $context = [],
        $print_r = false
    ) {
				if (empty($type)) $type = JLog::DEBUG;
        
        if (!empty($context) && is_array($context)) {
            $message .= ' Context: '. PHP_EOL;
            if ($print_r) {
                $message .= print_r($context, true) . PHP_EOL;
            }
            else {
                $message .= json_encode($context);
            }
        }
        
        JLog::add($message, $type, 'pm_vtbpay');
    }
}
